#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# IT WAS CREATED AND TESTED IN A WINDOWS ENVIRONMENT ONLY AND MAY NOT FUNCTION ON A MAC.
#------------------------------------------------------------------------------------------------
#  Name:	AddPts2Curve
#    By:	sdmitch
# Usage:	Add vertices to an existing curve at a given interval.
#  Date:	Jan 2012
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
if !file_loaded?(File.basename(__FILE__))
	UI.menu("Plugins").add_item("AddPts2Curve") { AddPts2Curve.do_curve}
	file_loaded(File.basename(__FILE__))
end#if

module AddPts2Curve
	
	def self.do_curve
		mod = Sketchup.active_model
		ent = mod.active_entities
		sel = mod.selection
		if sel.empty? or !sel.first.curve then
			UI.messagebox "No Curve selected";return
		end
		@pts_opt="Add Points" if !@pts_opt
		(@dinc=3.feet; @dinc=1.m if 	Sketchup.active_model.options["UnitsOptions"]["LengthUnit"] > 1) if !@dinc
		ans=UI.inputbox(["New Points Option:","Distance Increment:"],[@pts_opt,@dinc],["Add Points|Show Points"],"Add Points to Curve")
		if !ans then return end
		@pts_opt=ans[0]; @dinc=ans[1].to_l; curve=sel.first.curve; clen = curve.length;is_arc=false
		if @dinc < clen then
			mod.start_operation "Add Points to Curve"
			(ctr=curve.center;rad=curve.radius;is_arc=true) if curve.is_a?(Sketchup::ArcCurve)
			pts=curve.vertices.collect{|v| v.position}
			dist=@dinc; i = 1; npts=[];npts.push pts[0]
			if is_arc and UI.messagebox("Treat Arc like Curve?",MB_YESNO)==7
				ang=@dinc/rad
				tr=Geom::Transformation.rotation ctr,curve.normal,ang
				p0=ctr.offset(curve.xaxis,rad)
				ang.step(curve.end_angle,ang){|a|
					p0.transform! tr; 
					ent.add_cpoint(p0) if @pts_opt == "Show Points"
					npts.push p0.clone	#.clone required apparently because p0 changes via .transform!
				}
				npts.push pts[-1]
			else
				while i < pts.length
					d = npts[-1].distance(pts[i])
					v = npts[-1].vector_to(pts[i])
					if d < dist then
						npts.push pts[i]
						dist -= d
						i += 1
					elsif d == dist
						npts.push pts[i]
						dist = @dinc
						i += 1
					elsif d > dist
						p0=npts[-1].offset(v,dist)
						ent.add_cpoint(p0) if @pts_opt == "Show Points"
						npts.push p0	#.clone not required apparently because p0 value is assigned via =
						dist = @dinc
					end
				end
			end
			if @pts_opt == "Add Points" then
				ent.erase_entities(sel) if is_arc
				sel.clear; curve = ent.add_curve(npts); sel.add curve
			end
			mod.commit_operation
		else
			UI.messagebox("Distance entered is longer than the selected curve")
		end
	end
	
end#module
